using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebPartPages;
using System.Diagnostics;


namespace Microsoft.IW
{
	public class MySiteCreate : SPFeatureReceiver
	{

		public override void FeatureActivated(SPFeatureReceiverProperties properties)
		{
			try
			{
				string newMaster = string.Empty;

				//look at the properties collection to get the new master page name
				newMaster = properties.Feature.Properties["MasterName"].Value;

				if ((newMaster != null) && (newMaster != string.Empty))
				{
					using (SPWeb curWeb = (SPWeb)properties.Feature.Parent)
					{
						//got the current site and root web in site, now set the master Url
						//to our master page that should have been uploaded as part 
						//of our feature
						if (curWeb.MasterUrl.Contains("default.master"))
						{
							curWeb.MasterUrl = curWeb.MasterUrl.Replace("default.master", newMaster);
							curWeb.Update();
							UpdateLog("MasterUrl property updated for web " + curWeb.Title,
								EventLogEntryType.Information);
						}
					}
				}
			}
			catch (Exception ex)
			{
				//try writing to event log
				UpdateLog("Error in handler: " + ex.Message, EventLogEntryType.Error);
			}
		}


		private void UpdateLog(string Message, EventLogEntryType msgType)
		{
			try
			{
				System.Diagnostics.EventLog.WriteEntry("My Site Created Handler", Message, msgType);
			}
			catch
			{
				//ignore
			}
		}

		public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
		{
			//throw new Exception("The method or operation is not implemented.");
		}

		public override void FeatureInstalled(SPFeatureReceiverProperties properties)
		{
			//throw new Exception("The method or operation is not implemented.");
		}

		public override void FeatureUninstalling(SPFeatureReceiverProperties properties)
		{
			//throw new Exception("The method or operation is not implemented.");
		}
	}
}
