using System;
using System.Collections;
using System.Collections.Generic;
using System.Web.UI;
using System.Web.UI.WebControls;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebPartPages;
using System.Xml;
using System.Xml.Serialization;
using System.Xml.Schema;
using System.Diagnostics;
using System.Reflection;


namespace Microsoft.IW
{

	[XmlRoot("Microsoft.IW")]
	public class PartCheck : WebControl
	{

		//constants for property simple property setter value tag replacements
		public const string PSV_DOMAIN = "[DOMAIN]";
		public const string PSV_ACCOUNTNAME = "[ACCOUNTNAME]";
		public const string PSV_MACHINE = "[MACHINE]";
		public const string PSV_LOGIN = "[LOGIN]";
		//look to see if our web has had this stuff run yet
		private const string KEY_CHK = "Microsoft.IW.PartCheck";
	    private const string BLOG_CHK = "Microsoft.IW.PartCheck.BlogCheck";


		protected override void OnLoad(EventArgs e)
		{
			//do the base load functions
			base.OnLoad(e);


			//generic dictionary to store the list of webparts we're going to work 
			//with and what we'll do with each;
			Dictionary<string, WebPartAction> webPartList = null;

		    //webpart class that will be used in the action class
			WebPart webPart = null;

		    try
			{
				//get the current web; not using "using" because we don't want to kill
				//the web context for other controls that need it
				SPWeb curWeb = SPContext.Current.Web;

                if (curWeb.Properties.ContainsKey(BLOG_CHK))
                {
                    // Remove the create blog link
                    if (bool.Parse(curWeb.Properties[BLOG_CHK]))
		                RemoveCreateBlogButton();
                }
				//look to see if our code has already run
				if (!curWeb.Properties.ContainsKey(KEY_CHK))
				{
					//turn on unsafe updates so we can make changes on a GET
					curWeb.AllowUnsafeUpdates = true;

                    //look for the default page so we can mess with the web parts
                    SPFile thePage = curWeb.RootFolder.Files["default.aspx"];
                    if (thePage != null)
                    {
                        try
                        {
                            webPartList = Process(curWeb, thePage, webPartList, webPart);
                        }
                        finally
                        {
                            if (webPartList != null)
                            {
                                foreach (WebPartAction wpa1 in webPartList.Values)
                                {
                                    if (wpa1.wp != null)
                                        wpa1.wp.Dispose();
                                }
                            }
                            curWeb.AllowUnsafeUpdates = false;

                            //force a page refresh to show the page with the updated layout
                            Context.Response.Redirect(thePage.Url);
                        }
                    }
				}
			}
			catch (System.Threading.ThreadAbortException)
			{
				//we'll throw a thread abort exception on redirect, so we can ignore it, but
				//record everything else
			}
			catch (Exception ex)
			{
                UpdateLog("Error checking web parts: " + FormatException(ex), EventLogEntryType.Error);
			}

		}

        /// <summary>
        /// Processes the specified settings.
        /// </summary>
        /// <param name="curWeb">The current web.</param>
        /// <param name="thePage">The page.</param>
        /// <param name="webPartList">The wp list.</param>
        /// <param name="webPart">The x wp.</param>
        /// <returns></returns>
	    private Dictionary<string, WebPartAction> Process(SPWeb curWeb, SPFile thePage, Dictionary<string, WebPartAction> webPartList, WebPart webPart)
	    {
            //look for the xml manifest of web part changes
	        SPQuery qry = new SPQuery();
	        qry.Query = "<Where><Eq><FieldRef Name='FileLeafRef'/><Value Type='Text'>" +
	                    "MySiteStaplee.xml</Value></Eq></Where>";

	        //try opening the file
	        SPList xList = curWeb.GetCatalog(SPListTemplateType.MasterPageCatalog);
	        SPListItemCollection xItems = xList.GetItems(qry);

	        if ((xItems != null) && (xItems.Count > 0))
	        {
	            //create a hashtable to store our web parts
	            webPartList = new Dictionary<string,WebPartAction>();

	            //now that we have the file, read it into an xml document
	            XmlDocument xDoc = new XmlDocument();
	            xDoc.Load(xItems[0].File.OpenBinaryStream());
							
	            //create a new serializer
	            XmlSerializer slz = new XmlSerializer(typeof(WebPartAction));
	            
                bool removeBlogButton = false;
                if (xDoc.DocumentElement.Attributes["removeBlogButton"] != null)
                {
                    removeBlogButton = bool.Parse(xDoc.DocumentElement.Attributes["removeBlogButton"].Value);
                    if (removeBlogButton)
                        RemoveCreateBlogButton();
                }

	            //enumerate through the child nodes
	            foreach (XmlNode xNode in xDoc.FirstChild.ChildNodes)
	            {
	                WebPartAction wpa;
	                try
	                {
	                    //try serializing each one
	                    wpa = slz.Deserialize(new System.IO.StringReader(xNode.OuterXml)) as WebPartAction;
	                }
	                catch (Exception srlzEx)
	                {
	                    //some error logging here about invalid xml file
	                    UpdateLog("Error serializing MySiteStaplee.xml file: " + FormatException(srlzEx),
	                              EventLogEntryType.Error);

	                    //set our reference to null so we don't try and do something with
	                    //a version that was successfully serialized previously
	                    wpa = null;
	                }

	                //make sure we serialized, and also make sure this is an Add
	                if (wpa != null)
	                {
	                    switch (wpa.Action)
	                    {
	                        case WebPartAction.ActionType.Add:

	                            webPart = AddWebPart(wpa, webPartList, webPart);
	                            break;
	                        default:
	                            //for delete, move, close, or SetProperties we want to capture 
	                            //the type name and plug it into the hash array
	                            webPartList.Add(wpa.typeName, wpa);
	                            break;
	                    }
	                }
	            }

	            //get the web part manager
	            using (SPLimitedWebPartManager theMan = thePage.GetLimitedWebPartManager(System.Web.UI.WebControls.WebParts.PersonalizationScope.Shared))
	            {
	                foreach (WebPart wp in theMan.WebParts)
	                {
	                    //check each web part to see if matches our typeName
	                    if (webPartList.ContainsKey(wp.GetType().ToString()))
	                        webPartList[wp.GetType().ToString()].wp = wp;
	                }

	                PerformAction(theMan, webPartList);
	            }
	            //add our key to the property bag so we don't run our provisioning code again
                curWeb.Properties.Add(KEY_CHK, "true");
                curWeb.Properties.Add(BLOG_CHK, removeBlogButton.ToString());
                curWeb.Properties.Update();

	            //potentially delete our xml manifest file that tells us what to do
	            //would be cool if we could just remove ourselves from the page
	            //or maybe we stay there so we could be activated again in the future?? That
	            //would be an interesting idea...
	        }
	        return webPartList;
	    }

        /// <summary>
        /// Performs the action.
        /// </summary>
        /// <param name="theMan">The man.</param>
        /// <param name="wpList">The wp list.</param>
	    private static void PerformAction(SPLimitedWebPartManager theMan, Dictionary<string, WebPartAction> wpList)
	    {
            //now enumerate items in hash; can't do it in WebPart collection 
            //on SPLimitedWebPartManager or it fails
	        foreach (string key in wpList.Keys)
	        {
	            WebPartAction wpa = wpList[key];

	            switch (wpa.Action)
	            {
	                case WebPartAction.ActionType.Delete:
	                    theMan.DeleteWebPart(wpa.wp);
	                    break;
	                case WebPartAction.ActionType.Close:
	                    theMan.CloseWebPart(wpa.wp);
	                    break;
	                case WebPartAction.ActionType.Move:
	                    theMan.MoveWebPart(wpa.wp, wpa.zoneID, int.Parse(wpa.zoneIndex));
	                    theMan.SaveChanges(wpa.wp);
	                    break;
	                case WebPartAction.ActionType.Add:
	                    theMan.AddWebPart(wpa.wp, wpa.zoneID, int.Parse(wpa.zoneIndex));
	                    break;
	                case WebPartAction.ActionType.SetProperties:
	                    SetWebPartProperties(wpa, wpa.wp);
	                    theMan.SaveChanges(wpa.wp);
	                    break;
	            }
	        }
	    }

        /// <summary>
        /// Adds the web part.
        /// </summary>
        /// <param name="wpa">The wpa.</param>
        /// <param name="wpList">The wp list.</param>
        /// <param name="xWp">The x wp.</param>
        /// <returns></returns>
	    private static WebPart AddWebPart(WebPartAction wpa, Dictionary<string, WebPartAction> wpList, WebPart xWp)
	    {
            //now try loading the assembly
	        Assembly wpAsm;
	        try
	        {
	            wpAsm = Assembly.Load(wpa.assemblyName);
	        }
	        catch (Exception asmEx)
	        {
	            //some error logging here about invalid assembly name
	            UpdateLog("Error loading assembly name " + wpa.assemblyName +
                          ": " + FormatException(asmEx), EventLogEntryType.Error);

	            //set our reference to null so we don't try and do something with
	            //a version that was successfully loaded previously
	            wpAsm = null;
	        }

	        if (wpAsm != null)
	        {
	            //try creating an instance of the class
	            try
	            {
	                xWp = (WebPart)wpAsm.CreateInstance(wpa.className);
	            }
	            catch (Exception instEx)
	            {
	                //some error logging here about invalid class name
	                UpdateLog("Error creating instance of class " + wpa.className +
                              ": " + FormatException(instEx), EventLogEntryType.Error);

	                //set our reference to null so we don't try and do something with
	                //a version that was successfully loaded previously
	                if (xWp != null)
	                    xWp.Dispose();
	                xWp = null;
	            }
                                                
	            //plug it into our class
	            if (xWp != null)
	            {
	                //enumerate and add properties to part
	                SetWebPartProperties(wpa, xWp);

	                //add part to class
	                wpa.wp = xWp;
	            }
	        }

	        //add it to the hashtable
	        wpList.Add(Guid.NewGuid().ToString(), wpa);
	        return xWp;
	    }


	    /// <summary>
        /// Removes the create blog button.
        /// </summary>
        private void RemoveCreateBlogButton()
        {
            try
            {
                ContentPlaceHolder content = (ContentPlaceHolder)Page.Master.FindControl("PlaceHolderMiniConsole");
                if (content != null)
                {
                    Control ctrl = FindControl(content.Controls, "TBBCreateBlogButton");
                    if (ctrl != null)
                        ctrl.Visible = false;
                }
            }
            catch (Exception ex)
            {
                Page.Response.Write("<strong style=\"color:red;\">An error occured</strong>: " + ex.Message);
            }
        }

        /// <summary>
        /// Finds the control.
        /// </summary>
        /// <param name="ctls">The CTLS.</param>
        /// <param name="id">The id.</param>
        /// <returns></returns>
        private static Control FindControl(ControlCollection ctls, string id)
        {
            Control foundCtrl = null;
            foreach (Control ct in ctls)
            {
                if (ct.ID == id)
                    return ct;
                if (ct.HasControls())
                    foundCtrl = FindControl(ct.Controls, id);
                if (foundCtrl != null)
                    break;
            }
            return foundCtrl;
        }

        /// <summary>
        /// Sets the web part properties.
        /// </summary>
        /// <param name="wpa">The wpa.</param>
        /// <param name="xWp">The x wp.</param>
		private static void SetWebPartProperties(WebPartAction wpa, WebPart xWp)
		{
			//check to see if there are any properties; if there are zero it won't say
			//zero, it will say null (unlike vb.net)
			if (wpa.Properties == null)
				return;

			//enumerate and add properties to part
			for (int p = 0; p < wpa.Properties.Property.Length; p++)
			{
				try
				{
					xWp.GetType().GetProperty(wpa.Properties.Property[p].Key).SetValue(xWp,
						GetPropertySetterValue(wpa.Properties.Property[p].Value), null);
				}
				catch (Exception propEx)
				{
					//some error logging here about invalid property name
					UpdateLog("Error setting property value " +
						wpa.Properties.Property[p].Key + " to " +
						wpa.Properties.Property[p].Value +
                        ": " + FormatException(propEx), EventLogEntryType.Error);
				}
			}
		}

        /// <summary>
        /// Gets the property setter value.
        /// </summary>
        /// <param name="Value">The value.</param>
        /// <returns></returns>
		private static string GetPropertySetterValue(string Value)
		{
			string ret = Value;

			switch (Value)
			{
				case PSV_ACCOUNTNAME:
					ret = Environment.UserName;
					break;
				case PSV_DOMAIN:
					ret = Environment.UserDomainName;
					break;
				case PSV_MACHINE:
					ret = Environment.MachineName;
					break;
				case PSV_LOGIN:
					ret = Environment.UserDomainName + "\\" + Environment.UserName;
					break;
				default:
					break;
			}

			return ret;
		}

        /// <summary>
        /// Updates the log.
        /// </summary>
        /// <param name="Message">The message.</param>
        /// <param name="msgType">Type of the MSG.</param>
		private static void UpdateLog(string Message, EventLogEntryType msgType)
		{
			try
			{
				EventLog.WriteEntry("MySiteCreatePart", Message, msgType);
			}
			catch
			{
				//ignore
			}
		}


        /// <summary>
        /// Formats the exception.
        /// </summary>
        /// <param name="ex">The exception object.</param>
        /// <returns></returns>
        private static string FormatException(Exception ex)
        {
            if (ex == null) return "";

            string msg = "";

            msg += "\r\nError Type:      " + ex.GetType();
            msg += "\r\nError Message:   " + ex.Message; //Get the error message
            msg += "\r\nError Source:    " + ex.Source;  //Source of the message
            msg += "\r\nError TargetSite:" + ex.TargetSite; //Method where the error occurred
            foreach (DictionaryEntry de in ex.Data)
            {
                msg += "\r\n" + de.Key + ": " + de.Value;
            }
            msg += "\r\nError Stack Trace:\r\n" + ex.StackTrace; //Stack Trace of the error
            if (ex.InnerException != null)
                msg += "\r\nInner Exception:\r\n" + FormatException(ex.InnerException);

            return msg;
        }
	}


	public class WebPartAction
	{

		public enum ActionType
		{
			Add,
			Delete,
            Close,
			Move,
			SetProperties
		}

		[XmlIgnore()]
		public WebPart wp = null;

		[XmlElement(Form = XmlSchemaForm.Unqualified)]
		public string assemblyName = string.Empty;

		[XmlElement(Form = XmlSchemaForm.Unqualified)]
		public string className = string.Empty;

		[XmlElement(Form = XmlSchemaForm.Unqualified)]
		public string zoneID = string.Empty;

		//using a string to greatly simplify potential errors during deserialization
		[XmlElement(Form = XmlSchemaForm.Unqualified)]	
		public string zoneIndex = "0";

		[XmlElement(Form = XmlSchemaForm.Unqualified)]
		public string typeName = string.Empty;

		[XmlElement(Form = XmlSchemaForm.Unqualified)]
		public ActionType Action;

		[XmlElement(Form = XmlSchemaForm.Unqualified)]
		public PropertyRoot Properties;

		public WebPartAction()
		{
			//parameter-less constructor needed for serialization
		}

		public WebPartAction(WebPart wp, ActionType Action)
		{
			this.wp = wp;
			this.Action = Action;
		}

		public WebPartAction(WebPart wp, ActionType Action, string zoneID, string zoneIndex)
		{
			this.wp = wp;
			this.Action = Action;
			this.zoneID = zoneID;
			this.zoneIndex = zoneIndex;
		}

		public class PropertyRoot
		{
			[XmlElement(Form = XmlSchemaForm.Unqualified)]
			public OneProperty[] Property;
		}

		public class OneProperty
		{
			[XmlAttribute(Form = XmlSchemaForm.Unqualified)]
			public string Key = string.Empty;

			[XmlAttribute(Form = XmlSchemaForm.Unqualified)]
			public string Value = string.Empty;
		}

	}
}
